@@ -33,7 +33,7 @@ CORFIL *copy_url_corefile(CSOUND *, const char *, int);
 
 CORFIL *corfile_create_w(void)
 {
-    CORFIL *ans = (CORFIL*)malloc(sizeof(CORFIL));
+    CORFIL *ans = (CORFIL*) malloc(sizeof(CORFIL));
     ans->body = (char*)calloc(100,1);
     ans->len = 100;
     ans->p = 0;
@@ -43,7 +43,7 @@ CORFIL *corfile_create_w(void)
 CORFIL *corfile_create_r(const char *text)
 {
     //char *strdup(const char *);
-    CORFIL *ans = (CORFIL*)malloc(sizeof(CORFIL));
+    CORFIL *ans = (CORFIL*) malloc(sizeof(CORFIL));
     ans->body = strdup(text);
     ans->len = strlen(text)+1;
     ans->p = 0;
@@ -85,8 +85,8 @@ void corfile_puts(const char *s, CORFIL *f)
       }
     }
     if (n > 0) {
-      /* put the extra NUL chars to the end */
-      while(--n >= 0) {
+      /* put the extra NULL chars to the end */
+      while (--n >= 0) {
         char *new;
         f->body[f->p++] = '\0';
         if (UNLIKELY(f->p >= f->len)) {
@@ -250,11 +250,11 @@ CORFIL *copy_to_corefile(CSOUND *csound, const char *fname,
       corfile_puts(buffer, mm);
       memset(buffer, '\0', 1024);
     }
-#ifdef SCORE_PARSER
+    //#ifdef SCORE_PARSER
     if (fromScore) {
       corfile_puts("\n#exit\n", mm);
     }
-#endif
+    //#endif
     corfile_putc('\0', mm);     /* For use in bison/flex */
     corfile_putc('\0', mm);     /* For use in bison/flex */
     if (fromScore) corfile_flush(mm);
@@ -366,7 +366,7 @@ int main(void)
   res = curl_easy_perform(curl_handle);
 
   /* check for errors */
-  if(res != CURLE_OK) {
+  if (res != CURLE_OK) {
     fprintf(stderr, "curl_easy_perform() failed: %s\n",
             curl_easy_strerror(res));
   }
@@ -384,7 +384,7 @@ int main(void)
   /* cleanup curl stuff */
   curl_easy_cleanup(curl_handle);
 
-  if(chunk.memory)
+  if (chunk.memory)
     free(chunk.memory);
 
   /* we're done with libcurl, so clean it up */
