@@ -24,6 +24,10 @@
 #include "csoundCore.h"     /*                              LINEVENT.C      */
 #include <ctype.h>
 
+#ifdef MSVC
+#include <fcntl.h>
+#endif
+
 #include "linevent.h"
 
 #ifdef PIPES
@@ -86,9 +90,6 @@ void RTLineset(CSOUND *csound)      /* set up Linebuf & ready the input files */
     }
 #endif
 #define MODE ,0
-#if defined(MSVC)
-#define O_RDONLY _O_RDONLY
-#endif
     else
       if (UNLIKELY((csound->Linefd=open(O->Linename, O_RDONLY|O_NDELAY MODE)) < 0))
         csoundDie(csound, Str("Cannot open %s"), O->Linename);
@@ -193,10 +194,10 @@ void csoundInputMessageInternal(CSOUND *csound, const char *message)
       int extralloc = STA(Linep) + size - STA(Linebufend);
       // csound->Message(csound, "extralloc: %d %d %d\n",
       //                 extralloc, size, (int)(STA(Linebufend) - STA(Linep)));
-      // FIXME -- Coverity points out that this test isalways false
+      // FIXME -- Coverity points out that this test is always false
       // and n is never used
 #if 0
-      if ((n=linevent_alloc(csound, (STA(linebufsiz) + extralloc) ), 0) != 0) {
+      if ((n=linevent_alloc(csound, (STA(linebufsiz) + extralloc))) != 0) {
         csoundErrorMsg(csound, Str("LineBuffer Overflow - "
                                    "Input Data has been Lost"));
         return;
@@ -317,9 +318,9 @@ static void sensLine(CSOUND *csound, void *userData)
           cp = newcp - 1;
         } while (pcnt < PMAX);
         if (e.opcod =='f' && e.p[1]<FL(0.0)); /* an OK case */
-        else        /* check sufficient pfields */
-          if (UNLIKELY(pcnt < 3 && e.opcod != 'e')) {
-            csound->ErrorMsg(csound, Str("too few pfields"));
+        else  /* Check for sufficient pfields (0-based, opcode counted already). */
+          if (UNLIKELY(pcnt < 2 && e.opcod != 'e')) {
+            csound->ErrorMsg(csound, Str("too few pfields (%d)"), pcnt + 1);
             goto Lerr;
           }
         if (UNLIKELY(pcnt > 1 && e.p[2] < FL(0.0))) {
@@ -368,9 +369,9 @@ static void sensLine(CSOUND *csound, void *userData)
 /* send a lineevent from the orchestra -matt 2001/12/07 */
 
 static const char *errmsg_1 =
-  Str_noop("event: param 1 must be \"a\", \"i\", \"q\", \"f\", or \"e\"");
+  Str_noop("event: param 1 must be \"a\", \"i\", \"q\", \"f\", \"d\", or \"e\"");
 static const char *errmsg_2 =
-  Str_noop("event: string name is allowed only for \"i\" and \"q\" events");
+  Str_noop("event: string name is allowed only for \"i\", \"d\", and \"q\" events");
 
 int eventOpcode_(CSOUND *csound, LINEVENT *p, int insname, char p1)
 {
@@ -384,7 +385,8 @@ int eventOpcode_(CSOUND *csound, LINEVENT *p, int insname, char p1)
     else  opcod = p1;
 
     if (UNLIKELY((opcod != 'a' && opcod != 'i' && opcod != 'q' && opcod != 'f' &&
-                  opcod != 'e') /*|| ((STRINGDAT*) p->args[0])->data[1] != '\0'*/))
+                  opcod != 'e' && opcod != 'd')
+                 /*|| ((STRINGDAT*) p->args[0])->data[1] != '\0'*/))
       return csound->PerfError(csound, p->h.insdshead, "%s", Str(errmsg_1));
     evt.strarg = NULL; evt.scnt = 0;
     evt.opcod = opcod;
@@ -395,7 +397,7 @@ int eventOpcode_(CSOUND *csound, LINEVENT *p, int insname, char p1)
     /* IV - Oct 31 2002: allow string argument */
     if (evt.pcnt > 0) {
       if (insname) {
-        if (UNLIKELY(evt.opcod != 'i' && evt.opcod != 'q'))
+        if (UNLIKELY(evt.opcod != 'i' && evt.opcod != 'q' && opcod != 'd'))
           return csound->PerfError(csound, p->h.insdshead, "%s", Str(errmsg_2));
         evt.p[1] =  csound->strarg2insno(csound,
                                            ((STRINGDAT*) p->args[1])->data, 1);
@@ -411,6 +413,12 @@ int eventOpcode_(CSOUND *csound, LINEVENT *p, int insname, char p1)
       for (i = 2; i <= evt.pcnt; i++)
         evt.p[i] = *p->args[i];
     }
+
+    if(opcod == 'd') {
+      evt.opcod = 'i';
+      evt.p[1] *= -1;
+    }
+
     if (insert_score_event_at_sample(csound, &evt, csound->icurTime) != 0)
       return csound->PerfError(csound, p->h.insdshead,
                                Str("event: error creating '%c' event"),
@@ -443,7 +451,8 @@ int eventOpcodeI_(CSOUND *csound, LINEVENT *p, int insname, char p1)
          opcod = *((STRINGDAT*) p->args[0])->data;
     else opcod = p1;
     if (UNLIKELY((opcod != 'a' && opcod != 'i' && opcod != 'q' && opcod != 'f' &&
-                  opcod != 'e') /*|| ((STRINGDAT*) p->args[0])->data[1] != '\0'*/))
+                  opcod != 'e' && opcod != 'd')
+                 /*|| ((STRINGDAT*) p->args[0])->data[1] != '\0'*/))
       return csound->InitError(csound, "%s", Str(errmsg_1));
     evt.strarg = NULL; evt.scnt = 0;
     evt.opcod = opcod;
@@ -453,7 +462,7 @@ int eventOpcodeI_(CSOUND *csound, LINEVENT *p, int insname, char p1)
     /* IV - Oct 31 2002: allow string argument */
     if (evt.pcnt > 0) {
       if (insname) {
-        if (UNLIKELY(evt.opcod != 'i' && evt.opcod != 'q'))
+        if (UNLIKELY(evt.opcod != 'i' && evt.opcod != 'q' && opcod != 'd'))
           return csound->InitError(csound, "%s", Str(errmsg_2));
         evt.p[1] = csound->strarg2insno(csound,((STRINGDAT *)p->args[1])->data, 1);
         evt.strarg = NULL; evt.scnt = 0;
@@ -470,6 +479,11 @@ int eventOpcodeI_(CSOUND *csound, LINEVENT *p, int insname, char p1)
           evt.p[i] = *p->args[i];
       }
     }
+    if(opcod == 'd') {
+      evt.opcod = 'i';
+      evt.p[1] *= -1;
+    }
+
     if (opcod == 'f' && (int) evt.pcnt >= 2 && evt.p[2] <= FL(0.0)) {
       FUNC  *dummyftp;
       err = csound->hfgens(csound, &dummyftp, &evt, 0);
