@@ -158,7 +158,7 @@ static int alloc_event(CSOUND *csound, unsigned long kcnt, unsigned char *data,
     if (MF(nEvents) >= MF(maxEvents)) {
       MF(maxEvents) += (MF(maxEvents) >> 3);
       MF(maxEvents) = (MF(maxEvents) + 64) & (~63);
-      tmp = (midiEvent_t*) mrealloc(csound, MF(eventList),
+      tmp = (midiEvent_t*) csound->ReAlloc(csound, MF(eventList),
                                     sizeof(midiEvent_t) * MF(maxEvents));
       MF(eventList) = tmp;
       tmp = &(MF(eventList)[MF(nEvents)]);
@@ -183,7 +183,7 @@ static int alloc_tempo(CSOUND *csound, unsigned long kcnt, double tempoVal)
     if (MF(nTempo) >= MF(maxTempo)) {
       MF(maxTempo) += (MF(maxTempo) >> 3);
       MF(maxTempo) = (MF(maxTempo) + 64) & (~63);
-      tmp = (tempoEvent_t*) mrealloc(csound, MF(tempoList),
+      tmp = (tempoEvent_t*) csound->ReAlloc(csound, MF(tempoList),
                                      sizeof(tempoEvent_t) * MF(maxTempo));
       MF(tempoList) = tmp;
       tmp = &(MF(tempoList)[MF(nTempo)]);
@@ -558,6 +558,7 @@ int csoundMIDIFileOpen(CSOUND *csound, const char *name)
     /* open file */
     if (name == NULL || name[0] == '\0')
       return -1;
+    //if (*name==3) name++;       /* Because of ETX added bt readOptions */
     if (strcmp(name, "stdin") == 0)
       f = stdin;
     else {
@@ -626,7 +627,7 @@ int csoundMIDIFileOpen(CSOUND *csound, const char *name)
       timeCode = (timeCode << 8) | c;
     }
     /* allocate structure */
-    MIDIFILE = (void*) mcalloc(csound, sizeof(midiFile_t));
+    MIDIFILE = (void*) csound->Calloc(csound, sizeof(midiFile_t));
     /* calculate ticks per second or beat based on time code */
     if (timeCode < 1 || (timeCode >= 0x8000 && (timeCode & 0xFF) == 0)) {
       csound->Message(csound, Str(" *** invalid time code: %d\n"), timeCode);
@@ -652,7 +653,7 @@ int csoundMIDIFileOpen(CSOUND *csound, const char *name)
       MF(timeCode) *= (double) (timeCode & 0xFF);
     }
     /* initialise structure data */
-    MF(totalKcnt) = 0UL;
+    MF(totalKcnt) = csound->global_kcounter;
     MF(nEvents) = 0; MF(maxEvents) = 0;
     MF(nTempo) = 0; MF(maxTempo) = 0;
     MF(eventList) = (midiEvent_t*) NULL;
@@ -770,13 +771,14 @@ int csoundMIDIFileClose(CSOUND *csound)
 }
 
 /* midirecv.c, resets MIDI controllers on a channel */
-extern  void    midi_ctl_reset(CSOUND *csound, short chan);
+extern  void    midi_ctl_reset(CSOUND *csound, int16 chan);
 
 /* called by csoundRewindScore() to reset performance to time zero */
 
 void midifile_rewind_score(CSOUND *csound)
 {
     int i;
+    OPARMS *O = csound->oparms;
 
     if (MIDIFILE != NULL) {
       /* reset event index and tempo */
@@ -786,8 +788,14 @@ void midifile_rewind_score(CSOUND *csound)
       csound->MTrkend = csound->Mxtroffs = csound->Mforcdecs = 0;
       /* reset controllers on all channels */
       for (i = 0; i < MAXCHAN; i++)
-        midi_ctl_reset(csound, (short) i);
+        midi_ctl_reset(csound, (int16) i);
+    } else if (O->FMidiname != NULL) {
+      csound->MTrkend = 0;
+      if (csoundMIDIFileOpen(csound, O->FMidiname) != 0)
+        csound->Die(csound, Str("Failed to load MIDI file."));
+      O->FMidiin = 1;
     }
+    else csound->Warning(csound, Str("Cannot rewind MIDI score\n"));
 }
 
  /* ------------------------------------------------------------------------ */
@@ -797,9 +805,13 @@ void midifile_rewind_score(CSOUND *csound)
 int midiTempoOpcode(CSOUND *csound, MIDITEMPO *p)
 {
     if (MIDIFILE == NULL)
-      *(p->kResult) = (MYFLT) (60.0 / csound->beatTime);
+      *(p->kResult) = FL(60.0) *csound->esr / (MYFLT)(csound->ibeatTime);
     else
       *(p->kResult) = (MYFLT) MF(currentTempo);
     return OK;
 }
 
+int midiFileStatus(CSOUND *csound, MIDITEMPO *p){
+  *p->kResult = csound->oparms->FMidiin;
+  return OK;
+}
