@@ -282,11 +282,12 @@ int strcpy_opcode_p(CSOUND *csound, STRGET_OP *p)
 int strcat_opcode(CSOUND *csound, STRCAT_OP *p)
 {
     int size;
-    char *str1 = strdup(p->str1->data), *str2 = strdup(p->str2->data);
+    char *str1 = cs_strdup(csound, p->str1->data),
+         *str2 = cs_strdup(csound, p->str2->data);
 
     if (str1 == NULL || str2 == NULL){
-      free(str1);
-      free(str2);
+      csound->Free(csound,str1);
+      csound->Free(csound,str2);
       if (UNLIKELY(((OPDS*) p)->insdshead->pds != NULL))
       return csoundPerfError(csound, ((OPDS*)p)->insdshead, "NULL string \n");
       else return csoundInitError(csound, "NULL string \n");
@@ -315,8 +316,8 @@ int strcat_opcode(CSOUND *csound, STRCAT_OP *p)
     strncpy((char*) p->r->data,  str1, p->r->size-1);
     strcat((char*) p->r->data, str2);
 
-    free(str2);                 /* not needed anymore */
-    free(str1);
+    csound->Free(csound, str2);                 /* not needed anymore */
+    csound->Free(csound, str1);
     return OK;
 }
 
@@ -373,14 +374,14 @@ sprintf_opcode_(CSOUND *csound,
     }
 
 
-    strseg = malloc(siz);
+    strseg = csound->Malloc(csound, siz);
     i = 0;
 
     while (1) {
       if (UNLIKELY(i >= siz)) {
         // return StrOp_ErrMsg(p, "format string too long");
         siz *= 2;
-        strseg = realloc(strseg, siz);
+        strseg = csound->ReAlloc(csound, strseg, siz);
       }
       if (*fmt != '%' && *fmt != '\0') {
         strseg[i++] = *fmt++;
@@ -401,7 +402,7 @@ sprintf_opcode_(CSOUND *csound,
         maxChars = str->size - len;
         strseg[i] = '\0';
         if (UNLIKELY(numVals <= 0)) {
-          free(strseg);
+          csound->Free(csound, strseg);
           return StrOp_ErrMsg(p, Str("insufficient arguments for format"));
         }
         numVals--;
@@ -425,10 +426,14 @@ sprintf_opcode_(CSOUND *csound,
             int offs = outstring - str->data;
             str->data = csound->ReAlloc(csound, str->data,
                                  str->size  + 24);
+            if(str->data == NULL) {
+              return StrOp_ErrMsg(p, Str("memory allocation failure"));
+            }
             str->size += 24;
             maxChars += 24;
             outstring = str->data + offs;
             //printf("maxchars = %d  %s\n", maxChars, strseg);
+            //printf("size: %d \n",str->size);
 
           }
           n = snprintf(outstring, maxChars, strseg, (int) MYFLT2LRND(*parm));
@@ -443,14 +448,19 @@ sprintf_opcode_(CSOUND *csound,
         case 'g':
         case 'G':
 #ifdef HAVE_SNPRINTF
+          //printf("%d %d \n", str->size, strlen(str->data));
           if (strlen(strseg) + 24 > (unsigned)maxChars) {
             int offs = outstring - str->data;
             str->data = csound->ReAlloc(csound, str->data, str->size  + 13);
+            if(str->data == NULL) {
+              return StrOp_ErrMsg(p, Str("memory allocation failure"));
+            }
             str->size += 24;
             maxChars += 24;
             outstring = str->data + offs;
             //printf("maxchars = %d  %s\n", maxChars, strseg);
           }
+          //printf("%d %d \n", str->size, strlen(str->data));
           n = snprintf(outstring, maxChars, strseg, (double)*parm);
 #else
           n = sprintf(outstring, strseg, (double)*parm);
@@ -458,7 +468,7 @@ sprintf_opcode_(CSOUND *csound,
           break;
         case 's':
           if (((STRINGDAT*)parm)->data == str->data) {
-            free(strseg);
+            csound->Free(csound, strseg);
             return StrOp_ErrMsg(p, Str("output argument may not be "
                                        "the same as any of the input args"));
           }
@@ -467,6 +477,9 @@ sprintf_opcode_(CSOUND *csound,
             str->data = csound->ReAlloc(csound, str->data,
                                         str->size  + ((STRINGDAT*)parm)->size +
                                         strlen(strseg));
+           if(str->data == NULL){
+              return StrOp_ErrMsg(p, Str("memory allocation failure"));
+            }
             str->size += ((STRINGDAT*)parm)->size + strlen(strseg);
             maxChars += ((STRINGDAT*)parm)->size + strlen(strseg);
             outstring = str->data + offs;
@@ -474,16 +487,20 @@ sprintf_opcode_(CSOUND *csound,
           n = snprintf(outstring, maxChars, strseg, ((STRINGDAT*)parm)->data);
           break;
         default:
-          free(strseg);
+          csound->Free(csound, strseg);
           return StrOp_ErrMsg(p, Str("invalid format string"));
         }
         if (n < 0 || n >= maxChars) {
           /* safely detected excess string length */
             int offs = outstring - str->data;
             str->data = csound->ReAlloc(csound, str->data, maxChars*2);
+            if (str->data == NULL) {
+              return StrOp_ErrMsg(p, Str("memory allocation failure"));
+            }
             outstring = str->data + offs;
             str->size = maxChars*2;
-            maxChars += str->size;
+            // VL: Coverity says this is unused. (which is true)
+            // maxChars += str->size;
 
         }
         outstring += n;
@@ -502,17 +519,18 @@ sprintf_opcode_(CSOUND *csound,
         segwaiting++;
     }
     if (UNLIKELY(numVals > 0)) {
-      free(strseg);
+      csound->Free(csound, strseg);
       return StrOp_ErrMsg(p, Str("too many arguments for format"));
     }
-    free(strseg);
+    csound->Free(csound, strseg);
     return OK;
 }
 
 int sprintf_opcode(CSOUND *csound, SPRINTF_OP *p)
 {
-    int size = p->sfmt->size+ 10*((int) p->INOCOUNT);
-    if (p->r->data == NULL && p->r->size < size) {
+    int size = p->sfmt->size+ 18*((int) p->INOCOUNT);
+    //printf("%d %d \n", p->r->size, strlen(p->r->data));
+    if (p->r->data == NULL || p->r->size < size) {
       /* this 10 is 1n incorrect guess which is OK with numbers*/
       p->r->data = csound->Calloc(csound, size);
       p->r->size = size;
@@ -530,7 +548,7 @@ static CS_NOINLINE int printf_opcode_(CSOUND *csound, PRINTF_OP *p)
 {
     STRINGDAT buf;
     int   err;
-    buf.size = 3072;
+    buf.size = /*strlen(p->sfmt->data) +*/ 3072;
     buf.data = csound->Calloc(csound, buf.size);
 
     err = sprintf_opcode_(csound, p, &buf, (char*) p->sfmt->data, &(p->args[0]),
@@ -609,7 +627,7 @@ int strtod_opcode_p(CSOUND *csound, STRTOD_OP *p)
       return StrOp_ErrMsg(p, Str("empty string"));
     while (isblank(*s)) s++;
     if (UNLIKELY(*s == '\0'))
-      return StrOp_ErrMsg(p, Str("empty string"));
+     return StrOp_ErrMsg(p, Str("empty string"));
     x = cs_strtod(s, &tmp);
     if (UNLIKELY(*tmp != '\0'))
       return StrOp_ErrMsg(p, Str("invalid format"));
@@ -625,10 +643,10 @@ int strtod_opcode_S(CSOUND *csound, STRSET_OP *p)
     s = (char*) p->str->data;
     while (isblank(*s)) s++;
     if (UNLIKELY(*s == '\0'))
-      return StrOp_ErrMsg(p, Str("empty string"));
+     return StrOp_ErrMsg(p, Str("empty string"));
     x = cs_strtod(s, &tmp);
     if (UNLIKELY(*tmp != '\0'))
-      return StrOp_ErrMsg(p, Str("invalid format"));
+     return StrOp_ErrMsg(p, Str("invalid format"));
     *p->indx = (MYFLT) x;
 
     return OK;
