@@ -28,6 +28,7 @@
 #include <vector>
 
 using namespace std;
+using namespace csound;
 
 extern "C"
 {
@@ -36,6 +37,40 @@ extern "C"
 #include <lualib.h>
 }
 
+// These redefinitions are required because for Android,
+// LuaJIT is built with gcc for the GNU runtime library;
+// but LuaCsound is built with the NDK for the bionic runtime library.
+
+#if defined(__ANDROID__)
+extern "C"
+{
+    #undef stdin
+    FILE *stdin  = &__sF[0];
+    #undef stdout
+    FILE *stdout = &__sF[1];
+    #undef stderr
+    FILE *stderr = &__sF[2];
+    volatile int * __errno_location(void)
+    {
+        return __errno();
+    }
+    int _IO_getc(FILE *file_)
+    {
+        return getc(file_);
+    }
+    int _IO_putc(int char_, FILE *file_)
+    {
+        return putc(char_, file_);
+    }
+    int __isoc99_fscanf (FILE *stream, const char *format, ...)
+    {
+        va_list arg;
+        va_start (arg, format);
+        return vfscanf(stream, format, arg);
+    }
+}
+#endif
+
 /**
  * L U A   O P C O D E S   F O R   C S O U N D
  *
