@@ -22,6 +22,8 @@
  */
 #include "OpcodeBase.hpp"
 
+using namespace csound;
+
 #include <cmath>
 
 /**
@@ -48,19 +50,23 @@ public:
   // Inputs.
   MYFLT *kvelocity;
   MYFLT *irdb;
+  MYFLT *iuse0dbfs;
   // State.
   MYFLT ir;
   MYFLT im;
   MYFLT ib;
   MYFLT onedrms;
+  MYFLT dbfs;
   KAMPMIDID() :
     kamplitude(0),
     kvelocity(0),
     irdb(0),
+    iuse0dbfs(0),
     ir(0),
     im(0),
     ib(0),
-    onedrms(0)
+    onedrms(0),
+    dbfs(1)
   {}
   int init(CSOUND *csound)
   {
@@ -73,11 +79,14 @@ public:
       ib = MYFLT(127.0) / ( MYFLT(126.0) * std::sqrt(ir) ) -
         MYFLT(1.0) / MYFLT(126.0);
       im = ( MYFLT(1.0) - ib ) / MYFLT(127.0);
+      if (*iuse0dbfs != FL(0.0)) {
+          dbfs = csound->Get0dBFS(csound);
+      }
       return OK;
   }
   int kontrol(CSOUND *csound)
   {
-      *kamplitude = std::pow( (im * (*kvelocity + ib) ), MYFLT(2.0) ) * onedrms;
+      *kamplitude = dbfs * std::pow( (im * (*kvelocity + ib) ), MYFLT(2.0) ) * onedrms;
       return OK;
   }
 };
@@ -90,19 +99,23 @@ public:
   // Inputs.
   MYFLT *ivelocity;
   MYFLT *irdb;
+  MYFLT *iuse0dbfs;
   // State.
   MYFLT ir;
   MYFLT im;
   MYFLT ib;
   MYFLT onedrms;
+  MYFLT dbfs;
   IAMPMIDID() :
     iamplitude(0),
     ivelocity(0),
     irdb(0),
+    iuse0dbfs(0),
     ir(0),
     im(0),
     ib(0),
-    onedrms(0)
+    onedrms(0),
+    dbfs(1)
   {}
   int init(CSOUND *csound)
   {
@@ -115,7 +128,10 @@ public:
       ib = MYFLT(127.0) / ( MYFLT(126.0) * std::sqrt(ir) ) -
         MYFLT(1.0) / MYFLT(126.0);
       im = ( MYFLT(1.0) - ib ) / MYFLT(127.0);
-      *iamplitude = std::pow( (im * (*ivelocity + ib) ), MYFLT(2.0) ) * onedrms;
+      if (*iuse0dbfs != FL(0.0)) {
+          dbfs = csound->Get0dBFS(csound);
+      }
+      *iamplitude = dbfs * std::pow( (im * (*ivelocity + ib) ), MYFLT(2.0) ) * onedrms;
       return OK;
   }
   int noteoff(CSOUND *)
@@ -126,7 +142,7 @@ public:
 
 extern "C" {
 
-#ifndef PNACL  
+#ifndef PNACL
   PUBLIC int csoundModuleCreate(CSOUND *csound)
   {
       return 0;
@@ -141,7 +157,7 @@ extern "C" {
                                         0,
                                         3,
                                         (char*)"k",
-                                        (char*)"ki",
+                                        (char*)"kio",
                                         (int(*)(CSOUND*,void*)) KAMPMIDID::init_,
                                         (int(*)(CSOUND*,void*)) KAMPMIDID::kontrol_,
                                         (int (*)(CSOUND*,void*)) 0);
@@ -151,7 +167,7 @@ extern "C" {
                                         0,
                                      1,
                                      (char*)"i",
-                                     (char*)"ii",
+                                     (char*)"iio",
                                      (int (*)(CSOUND*,void*)) IAMPMIDID::init_,
                                      (int (*)(CSOUND*,void*)) 0,
                                      (int (*)(CSOUND*,void*)) 0);
@@ -168,7 +184,7 @@ extern "C" {
       return status;
   }
 
-#ifndef PNACL  
+#ifndef PNACL
   PUBLIC int csoundModuleInit(CSOUND *csound)
   {
       return csoundModuleInit_ampmidid(csound);
@@ -181,6 +197,3 @@ extern "C" {
   }
 #endif
 }
-
-
-
