@@ -18,7 +18,12 @@ ldd_globs = '*.exe *.dll *.so *.node *.pyd'
 ldd_filepath = r'D:\msys64\usr\bin\ldd'
 
 def exclude(filepath):
-    if fnmatch.fnmatch(filepath, '''*/examples/*'''):
+    # Wrong case!
+    if filepath.endswith('CSOUND64.dll'):
+        return True
+    if fnmatch.fnmatch(filepath, '''*/python27.dll'''):
+        return True
+    if fnmatch.fnmatch(filepath, '''*/android/*'''):
         return True
     if fnmatch.fnmatch(filepath, '''*/frontends/*'''):
         return True
@@ -105,27 +110,33 @@ targets = set()
 dependencies = set()
 # In order to identify what file some dependency is for.
 with open('mingw64/csound_ldd.txt', 'w') as f:
+    print 'f:', f
     for dirpath, dirnames, files in os.walk('.'):
         for glob in globs.split(' '):
             matches = fnmatch.filter(files, glob)
             for match in matches:
                 filepath = os.path.join(dirpath, match)
+                print 'filepath:', filepath
                 if filepath.find('Setup_Csound6_') == -1:
                     targets.add(filepath)
     for target in sorted(targets):
         dependencies.add(target)
-        if fnmatch.fnmatch(target, ldd_globs):
-            popen = subprocess.Popen([ldd_filepath, target], stdout = subprocess.PIPE)
-            output = popen.communicate()[0]
-            f.write(target + '\n')
-            if len(output) > 1:
-                f.write(output + '\n')
-            for line in output.split('\n'):
-                parts = line.split()
-                if len(parts) > 2:
-                    dependencies.add(parts[2])
-                elif len(parts) > 0:
-                dependencies.add(parts[0])
+        print 'target:', target
+        for ldd_glob in ldd_globs.split():
+            if fnmatch.fnmatch(target, ldd_glob):
+                print 'match: ', target
+                popen = subprocess.Popen([ldd_filepath, target], stdout = subprocess.PIPE)
+                output = popen.communicate()[0]
+                print 'output:', output
+                f.write(target + '\n')
+                if len(output) > 1:
+                    f.write(output + '\n')
+                for line in output.split('\n'):
+                    parts = line.split()
+                    if len(parts) > 2:
+                        dependencies.add(parts[2])
+                    elif len(parts) > 0:
+                        dependencies.add(parts[0])
 print
 print 'CSOUND TARGETS AND DEPENDENCIES'
 print
@@ -133,14 +144,19 @@ dependencies = sorted(dependencies)
 nonsystem_dependencies = set()
 for dependency in dependencies:
     realpath = os.path.abspath(dependency)
+    print 'realpath:', realpath
     # Fix up MSYS pathname confusion.
-    realpath = realpath.replace('''D:/''', '''D:/msys64/''')
-    realpath = realpath.replace('''D:/msys64/msys64/''', '''D:/msys64/''')
+    realpath = realpath.replace('\\home\\restore\\', '\\msys64\\home\\restore\\')
+    realpath = realpath.replace('D:\\c\\', 'C:\\')
+    realpath = realpath.replace('D:\\', 'D:\\msys64\\')
+    realpath = realpath.replace('D:\\msys64\\msys64\\', 'D:\\msys64\\')
+    realpath = realpath.replace('D:\\msys64\\msys64\\', 'D:\\msys64\\')
+    print 'fixed   :', realpath
     nonsystem_dependencies.add(realpath)
 nonsystem_dependencies = sorted(nonsystem_dependencies)
 with open('installer/windows/csound_targets_and_dependencies.iss', 'w') as f:
     for dependency in nonsystem_dependencies:
         if not exclude(dependency):
-            print dependency
+            print 'dependency:', dependency
             line = emit(dependency)
             f.write(line)
