@@ -5,19 +5,20 @@
 
     This file is part of Csound.
 
-    Csound is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    The Csound Library is free software; you can redistribute it
+    and/or modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
     Csound is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with Csound; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+    02111-1307 USA
 */
 
 /*******************************************************\
@@ -27,6 +28,7 @@
 *   including lifting from Csound itself                *
 \*******************************************************/
 
+
 /* Notes:
  *     This makes a mess of multichannel inputs.
  *     Needs to take much more care
@@ -64,7 +66,7 @@ typedef struct inputs {
     scalepoint *fulltable;      /* Scaling table */
     scalepoint *table;          /* current position in table */
     SNDFILE    *fd;             /* File descriptor handle */
-    short       channels[5];    /* destinations of channels */
+    int16       channels[5];    /* destinations of channels */
     int         non_clear;      /* Boolean to say if fiddled mixing */
     SOUNDIN *   p;              /* Csound structure */
 } inputs;
@@ -84,35 +86,35 @@ typedef struct mixer_globals_ {
 static  void    InitScaleTable(MIXER_GLOBALS *, int);
 static  MYFLT   gain(MIXER_GLOBALS *, int, int);
 static  SNDFILE *MXsndgetset(CSOUND*,inputs *);
-static  void    MixSound(MIXER_GLOBALS *, int, SNDFILE *);
+static  void    MixSound(MIXER_GLOBALS *, int, SNDFILE *, OPARMS *);
 
 static const char *usage_txt[] = {
-    "Usage:\tmixer [-flags] soundfile [-flags] soundfile ...",
-    "Legal flags are:",
-    "-o fnam\tsound output filename",
-    "-A\tcreate an AIFF format output soundfile",
-    "-W\tcreate a WAV format output soundfile",
-    "-h\tno header on output soundfile",
-    "-8\t8-bit unsigned_char sound samples",
-    "-c\t8-bit signed_char sound samples",
-    "-8\t8-bit unsigned_char sound samples",
-    "-a\talaw sound samples",
-    "-u\tulaw sound samples",
-    "-s\tshort_int sound samples",
-    "-l\tlong_int sound samples",
-    "-f\tfloat sound samples",
-    "-R\tcontinually rewrite header while writing soundfile (WAV/AIFF)",
-    "-H#\tprint a heartbeat style 1, 2 or 3 at each soundfile write",
-    "-N\tnotify (ring the bell) when score or miditrack is done",
-    "-F fpnum\tamount to scale amplitude for next input",
-    "-F fname\tfile of a scale table for next input",
-    "-S integer\tsample number at which to insert file",
-    "-T fpnum\ttime at which to insert file",
-    "-1 -2 -3 -4\tinclude named channel",
-    "-^ n m\tinclude channel n and output as channel m",
-    "-v\tverbose mode for debugging",
-    "-- fname\tLog output to file",
-    "flag defaults: mixer -s -otest -F 1.0 -S 0",
+  Str_noop("Usage:\tmixer [-flags] soundfile [-flags] soundfile ..."),
+  Str_noop("Legal flags are:"),
+  Str_noop("-o fnam\tsound output filename"),
+  Str_noop("-A\tcreate an AIFF format output soundfile"),
+  Str_noop("-W\tcreate a WAV format output soundfile"),
+  Str_noop("-h\tno header on output soundfile"),
+  Str_noop("-8\t8-bit unsigned_char sound samples"),
+  Str_noop("-c\t8-bit signed_char sound samples"),
+  Str_noop("-8\t8-bit unsigned_char sound samples"),
+  Str_noop("-a\talaw sound samples"),
+  Str_noop("-u\tulaw sound samples"),
+  Str_noop("-s\tshort_int sound samples"),
+  Str_noop("-l\tlong_int sound samples"),
+  Str_noop("-f\tfloat sound samples"),
+  Str_noop("-R\tcontinually rewrite header while writing soundfile (WAV/AIFF)"),
+  Str_noop("-H#\tprint a heartbeat style 1, 2 or 3 at each soundfile write"),
+  Str_noop("-N\tnotify (ring the bell) when score or miditrack is done"),
+  Str_noop("-F fpnum\tamount to scale amplitude for next input"),
+  Str_noop("-F fname\tfile of a scale table for next input"),
+  Str_noop("-S integer\tsample number at which to insert file"),
+  Str_noop("-T fpnum\ttime at which to insert file"),
+  Str_noop("-1 -2 -3 -4\tinclude named channel"),
+  Str_noop("-^ n m\tinclude channel n and output as channel m"),
+  Str_noop("-v\tverbose mode for debugging"),
+  Str_noop("-- fname\tLog output to file"),
+  Str_noop("flag defaults: mixer -s -otest -F 1.0 -S 0"),
     NULL
 };
 
@@ -130,9 +132,8 @@ static void usage(CSOUND *csound, const char *mesg, ...)
     csound->LongJmp(csound, 1);
 }
 
-static char set_output_format(CSOUND *csound, char c, char outformch)
+static char set_output_format(CSOUND *csound, char c, char outformch, OPARMS *O)
 {
-    OPARMS  *O = csound->oparms;
 
     switch (c) {
       case 'a': O->outformat = AE_ALAW;   /* a-law soundfile */
@@ -160,9 +161,9 @@ static char set_output_format(CSOUND *csound, char c, char outformch)
 
 static int mixer_main(CSOUND *csound, int argc, char **argv)
 {
-    OPARMS      *O = csound->oparms;
+    OPARMS      O;
     char        *inputfile = NULL;
-    SNDFILE     *infd, *outfd;
+    SNDFILE     *outfd;
     int         i;
     char        outformch='s', c, *s;
     const char  *envoutyp;
@@ -172,16 +173,18 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
                                                         sizeof(MIXER_GLOBALS));
     inputs      *mixin = &(pp->mixin[0]);
 
+    csound->GetOParms(csound, &O);
+
     pp->csound = csound;
-    csound->dbfs_to_float = csound->e0dbfs = FL(1.0);
+    /*csound->dbfs_to_float = csound->e0dbfs = FL(1.0);*/
     /* Check arguments */
     if ((envoutyp = csound->GetEnv(csound, "SFOUTYP")) != NULL) {
       if (strcmp(envoutyp, "AIFF") == 0)
-        O->filetyp = TYP_AIFF;
+        O.filetyp = TYP_AIFF;
       else if (strcmp(envoutyp, "WAV") == 0)
-        O->filetyp = TYP_WAV;
+        O.filetyp = TYP_WAV;
       else if (strcmp(envoutyp, "IRCAM") == 0)
-        O->filetyp = TYP_IRCAM;
+        O.filetyp = TYP_IRCAM;
       else {
         csound->ErrorMsg(csound, Str("%s not a recognized SFOUTYP env setting"),
                                  envoutyp);
@@ -201,25 +204,24 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
           switch(c) {
           case 'o':
             FIND(Str("no outfilename"))
-            O->outfilename = s;         /* soundout name */
+            O.outfilename = s;         /* soundout name */
             for ( ; *s != '\0'; s++) ;
-            if (strcmp(O->outfilename, "stdin") == 0)
+            if (strcmp(O.outfilename, "stdin") == 0)
               csound->Die(csound, Str("mixer: -o cannot be stdin"));
-#if defined(mac_classic) || defined(SYMANTEC) || defined(BCC) ||  \
-    defined(__WATCOMC__) || defined(WIN32)
-            if (strcmp(O->outfilename,"stdout") == 0) {
+#if defined(WIN32)
+            if (strcmp(O.outfilename,"stdout") == 0) {
               csound->Die(csound, Str("mixer: stdout audio not supported"));
             }
 #endif
             break;
           case 'A':
-            O->filetyp = TYP_AIFF;      /* AIFF output request  */
+            O.filetyp = TYP_AIFF;      /* AIFF output request  */
             break;
           case 'J':
-            O->filetyp = TYP_IRCAM;     /* IRCAM output request */
+            O.filetyp = TYP_IRCAM;     /* IRCAM output request */
             break;
           case 'W':
-            O->filetyp = TYP_WAV;       /* WAV output request  */
+            O.filetyp = TYP_WAV;       /* WAV output request  */
             break;
           case 'F':
             FIND(Str("no scale factor"));
@@ -229,7 +231,7 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
               mixin[n].fname = (char*) csound->Malloc(csound, strlen(s) + 1);
               strcpy(mixin[n].fname, s);
               mixin[n].use_table = 1;
-            }
+           }
             while (*++s);
             break;
           case 'S':
@@ -282,7 +284,7 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
               break;
             }
           case 'h':
-            O->filetyp = TYP_RAW;       /* skip sfheader  */
+            O.filetyp = TYP_RAW;       /* skip sfheader  */
             break;
           case 'c':
           case 'a':
@@ -291,21 +293,21 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
           case 's':
           case 'l':
           case 'f':
-            outformch = set_output_format(csound, c, outformch);
+            outformch = set_output_format(csound, c, outformch, &O);
             break;
           case 'R':
-            O->rewrt_hdr = 1;
+            O.rewrt_hdr = 1;
             break;
           case 'H':
             if (isdigit(*s)) {
               int n;
-              sscanf(s, "%d%n", &O->heartbeat, &n);
+              sscanf(s, "%d%n", &O.heartbeat, &n);
               s += n;
             }
-            else O->heartbeat = 1;
+            else O.heartbeat = 1;
             break;
           case 'N':
-            O->ringbell = 1;             /* notify on completion */
+            O.ringbell = 1;             /* notify on completion */
             break;
           case 'v':                       /* Verbose mode */
             pp->debug = 1;
@@ -318,7 +320,7 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
         mixin[n].name = --s;
         if (!mixin[n].non_clear)
           for (i=1; i<5; i++) mixin[n].channels[i] = i;
-        if (n++ >= NUMBER_OF_FILES) {
+        if (UNLIKELY(n++ >= NUMBER_OF_FILES-1)) {
           usage(csound,Str("Too many mixin"));
         }
         mixin[n].start = -1;
@@ -330,11 +332,11 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
 
     /* Read sound files */
     if (n == 0) {
-      csound->ErrorMsg(csound, "No mixin");
+      csound->ErrorMsg(csound, Str("No mixin"));
       return -1;
     }
     for (i = 0; i < n; i++) {
-      if (!(infd = MXsndgetset(csound, &mixin[i]))) {
+      if (!MXsndgetset(csound, &mixin[i])) {
         csound->ErrorMsg(csound, Str("%s: error while opening %s"),
                                  argv[0], inputfile);
         return -1;
@@ -364,31 +366,31 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
       if (mixin[i].use_table) InitScaleTable(pp, i);
     }
 
-    if (!O->outformat)                      /* if no audioformat yet  */
-      O->outformat = mixin[0].p->format;    /* Copy from first input file */
-    O->sfsampsize = csound->sfsampsize(FORMAT2SF(O->outformat));
-    if (!O->filetyp)
-      O->filetyp = mixin[0].p->filetyp;     /* Copy from input file */
-    O->sfheader = (O->filetyp == TYP_RAW ? 0 : 1);
-    if (!O->sfheader)       /* can't rewrite header if no header requested */
-      O->rewrt_hdr = 0;
+    if (!O.outformat)                      /* if no audioformat yet  */
+      O.outformat = mixin[0].p->format;    /* Copy from first input file */
+    O.sfsampsize = csound->sfsampsize(FORMAT2SF(O.outformat));
+    if (!O.filetyp)
+      O.filetyp = mixin[0].p->filetyp;     /* Copy from input file */
+    O.sfheader = (O.filetyp == TYP_RAW ? 0 : 1);
+    if (!O.sfheader)       /* can't rewrite header if no header requested */
+      O.rewrt_hdr = 0;
 #ifdef NeXT
-    if (O->outfilename == NULL && !O->filetyp) O->outfilename = "test.snd";
-    else if (O->outfilename == NULL) O->outfilename = "test";
+    if (O.outfilename == NULL && !O.filetyp) O.outfilename = "test.snd";
+    else if (O.outfilename == NULL) O.outfilename = "test";
 #else
-    if (O->outfilename == NULL) {
-      if (O->filetyp == TYP_WAV) O->outfilename = "test.wav";
-      else if (O->filetyp == TYP_AIFF) O->outfilename = "test.aif";
-      else O->outfilename = "test";
+    if (O.outfilename == NULL) {
+      if (O.filetyp == TYP_WAV) O.outfilename = "test.wav";
+      else if (O.filetyp == TYP_AIFF) O.outfilename = "test.aif";
+      else O.outfilename = "test";
     }
 #endif
-    csound->esr = (MYFLT) mixin[0].p->sr;
+    csound->SetUtilSr(csound, (MYFLT)mixin[0].p->sr);
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sfinfo.frames = -1;
-    sfinfo.samplerate = (int) MYFLT2LRND(csound->esr);
-    sfinfo.channels = csound->nchnls = (int) mixin[0].p->nchanls;
-    sfinfo.format = TYPE2SF(O->filetyp) | FORMAT2SF(O->outformat);
-    if (strcmp(O->outfilename, "stdout") == 0) {
+    sfinfo.samplerate = mixin[0].p->sr;
+    sfinfo.channels /*= csound->nchnls*/ = (int) mixin[0].p->nchanls;
+    sfinfo.format = TYPE2SF(O.filetyp) | FORMAT2SF(O.outformat);
+    if (strcmp(O.outfilename, "stdout") == 0) {
       outfd = sf_open_fd(1, SFM_WRITE, &sfinfo, 0);
       if (outfd != NULL) {
         if (csound->CreateFileHandle(csound,
@@ -398,27 +400,27 @@ static int mixer_main(CSOUND *csound, int argc, char **argv)
         }
       }
     }
-    else if (csound->FileOpen2(csound, &outfd, CSFILE_SND_W, O->outfilename,
-                       &sfinfo, "SFDIR", csound->type2csfiletype(O->filetyp,
-                       O->outformat), 0) == NULL)
+    else if (csound->FileOpen2(csound, &outfd, CSFILE_SND_W, O.outfilename,
+                       &sfinfo, "SFDIR", csound->type2csfiletype(O.filetyp,
+                       O.outformat), 0) == NULL)
       outfd = NULL;
     if (outfd == NULL) {
       csound->ErrorMsg(csound, Str("mixer: error opening output file '%s'"),
-                               O->outfilename);
+                               O.outfilename);
       return -1;
     }
-    if (O->rewrt_hdr)
+    if (O.rewrt_hdr)
       sf_command(outfd, SFC_SET_UPDATE_HEADER_AUTO, NULL, 0);
     /* calc outbuf size & alloc bufspace */
     pp->outbufsiz = NUMBER_OF_SAMPLES * pp->outputs;
     pp->out_buf = csound->Malloc(csound, pp->outbufsiz * sizeof(MYFLT));
-    pp->outbufsiz *= O->sfsampsize;
+    pp->outbufsiz *= O.sfsampsize;
     csound->Message(csound, Str("writing %d-byte blks of %s to %s (%s)\n"),
                             pp->outbufsiz,
-                            csound->getstrformat(O->outformat), O->outfilename,
-                            csound->type2string(O->filetyp));
-    MixSound(pp, n, outfd);
-    if (O->ringbell)
+                            csound->getstrformat(O.outformat), O.outfilename,
+                            csound->type2string(O.filetyp));
+    MixSound(pp, n, outfd, &O);
+    if (O.ringbell)
       csound->MessageS(csound, CSOUNDMSG_REALTIME, "\007");
     return 0;
 }
@@ -454,8 +456,11 @@ InitScaleTable(MIXER_GLOBALS *pp, int i)
       newpoint->x1 = (int) (x*samplepert);
       newpoint->y1 = y;
       if (newpoint->x1 == newpoint->x0) {
+        MYFLT div = (MYFLT)(tt->x1 - tt->x0);
         tt->y1 = y;
-        tt->yr = (y - tt->y0)/((MYFLT)(tt->x1 - tt->x0));
+        if (LIKELY(div))
+          tt->yr = (y - tt->y0)/div;
+        else  tt->yr = y;
         csound->Free(csound, newpoint);
       }
       else {
@@ -501,7 +506,7 @@ static MYFLT gain(MIXER_GLOBALS *pp, int n, int i)
     if (i<mixin[n].table->x0) mixin[n].table = mixin[n].fulltable;
     while (i<mixin[n].table->x0 ||
            i>=mixin[n].table->x1) {/* Get correct segment */
-      if (pp->debug)
+      if (UNLIKELY(pp->debug))
         csound->Message(csound, "Table %d: %d (%d %f) -> %d %f [%f]\n",
                         n, i, mixin[n].table->x0, mixin[n].table->y0,
                         mixin[n].table->x1, mixin[n].table->y1,
@@ -518,14 +523,14 @@ static SNDFILE *MXsndgetset(CSOUND *csound, inputs *ddd)
     MYFLT   dur;
     SOUNDIN *p;
 
-    csound->esr = FL(0.0);      /* set esr 0. with no orchestra   */
+    csound->SetUtilSr(csound, FL(0.0));         /* set esr 0. with no orchestra   */
     ddd->p = p = (SOUNDIN *) csound->Calloc(csound, sizeof(SOUNDIN));
     p->analonly = 1;
     p->channel = ALLCHNLS;
     p->skiptime = FL(0.0);
-    strcpy(p->sfname, ddd->name);
+    strncpy(p->sfname, ddd->name, MAXSNDNAME-1);
     /* open sndfil, do skiptime */
-    if ((infd = csound->sndgetset(csound, p)) == NULL)
+    if (UNLIKELY((infd = csound->sndgetset(csound, p)) == NULL))
       return NULL;
     p->getframes = p->framesrem;
     dur = (MYFLT) p->getframes / p->sr;
@@ -535,10 +540,9 @@ static SNDFILE *MXsndgetset(CSOUND *csound, inputs *ddd)
     return infd;
 }
 
-static void MixSound(MIXER_GLOBALS *pp, int n, SNDFILE *outfd)
+ static void MixSound(MIXER_GLOBALS *pp, int n, SNDFILE *outfd, OPARMS *O)
 {
     CSOUND *csound = pp->csound;
-    OPARMS  *O = csound->oparms;
     inputs  *mixin = &(pp->mixin[0]);
     MYFLT   *buffer = (MYFLT*) csound->Calloc(csound, sizeof(MYFLT)
                                                       * 6 * NUMBER_OF_SAMPLES);
@@ -567,12 +571,15 @@ static void MixSound(MIXER_GLOBALS *pp, int n, SNDFILE *outfd)
       for (i = 0; i < n; i++)
         if (mixin[i].start > sample && mixin[i].start - sample < size)
           size = mixin[i].start - sample;
-      for (j=0; j<size*outputs; j++) buffer[j] = FL(0.0);
+      /* for (j=0; j<size*outputs; j++) buffer[j] = FL(0.0); */
+      memset(buffer, 0, sizeof(MYFLT)*size*outputs);
       this_block = 0;
       for (i = 0; i<n; i++) {
         if (sample >= mixin[i].start) {
           read_in = csound->getsndin(csound, mixin[i].fd, ibuffer,
                                      size*mixin[i].p->nchanls, mixin[i].p);
+          for(i=0; i < read_in; i++)
+            ibuffer[i] *= 1.0/csound->Get0dBFS(csound);
           read_in /= mixin[i].p->nchanls;
           if (read_in > this_block) this_block = read_in;
           if (mixin[i].non_clear) {
@@ -604,7 +611,7 @@ static void MixSound(MIXER_GLOBALS *pp, int n, SNDFILE *outfd)
           more_to_read++;
       }
       for (j = 0; j < this_block * outputs; j++) {
-        if (buffer[j] > csound->e0dbfs || buffer[j] < -(csound->e0dbfs))
+        if (UNLIKELY(buffer[j] > 1.0 || buffer[j] < -(1.0)))
           pp->outrange++;
         if (buffer[j] == max) maxtimes++;
         if (buffer[j] == min) mintimes++;
@@ -634,9 +641,9 @@ static void MixSound(MIXER_GLOBALS *pp, int n, SNDFILE *outfd)
       }
       sample += size;
     }
-    csound->rewriteheader(outfd);
-    min *= (DFLT_DBFS * csound->dbfs_to_float);
-    max *= (DFLT_DBFS * csound->dbfs_to_float);
+    csound->rewriteheader((struct SNFDILE*)outfd);
+    min *= (DFLT_DBFS);
+    max *= (DFLT_DBFS);
     csound->Message(csound, Str("Max val %d at index %ld (time %.4f, chan %d) "
                                 "%d times\n"),
                             (int) max, lmaxpos, tpersample * (lmaxpos/outputs),
@@ -645,7 +652,7 @@ static void MixSound(MIXER_GLOBALS *pp, int n, SNDFILE *outfd)
                                 "%d times\n"),
                             (int) min, lminpos, tpersample * (lminpos/outputs),
                             (int) lminpos % outputs, mintimes);
-    if (pp->outrange)
+    if (UNLIKELY(pp->outrange))
       csound->Message(csound, Str("%d sample%s out of range\n"),
                               pp->outrange, (pp->outrange == 1 ? "" : "s"));
     else
@@ -660,10 +667,9 @@ int mixer_init_(CSOUND *csound)
     char    buf[128];
     int     retval = csound->AddUtility(csound, "mixer", mixer_main);
 
-    sprintf(buf, "Mixes sound files (max. %d)", (int) NUMBER_OF_FILES);
+    snprintf(buf, 128, Str("Mixes sound files (max. %d)"), (int) NUMBER_OF_FILES);
     if (!retval) {
       retval = csound->SetUtilityDescription(csound, "mixer", buf);
     }
     return retval;
 }
-
